/*
 * db.cpp
 *
 *  Created on: 2009/10/09
 *      Author: shu
 */


#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <stdint.h>
#include <limits.h>
#include <stdexcept>
#include "chunk.h"
#include "db.h"

using namespace std;

Db::Db(string filename_prefix)
: filename_prefix_(filename_prefix), number_chunks_(0), chunk_id_(UINT_MAX), chunk_(NULL)
{
  ifstream in;
  string filename = GetInformationFileName(filename_prefix_);
  in.open(filename.c_str(), ios::binary);
  if (in) {
    in.read((char *) &number_chunks_, sizeof(number_chunks_));
    in.read((char *) &max_length_, sizeof(max_length_));
    in.read((char *) &db_length_, sizeof(db_length_));
    in.read((char *) &number_sequences_, sizeof(number_sequences_));
    in.close();
  }
  SetChunk(0);
}


Db::~Db() {
  if (chunk_) {
    delete chunk_;
  }
}

void Db::SetChunk(uint32_t id) {
  if (id >= number_chunks_) {
    printf("id:%d chunk:%d\n", id, number_chunks_);
    throw out_of_range("id is invalid");
  }
  if (id == chunk_id_) {
    return;
  }
  if (chunk_) {
    delete chunk_;
    chunk_ = NULL;
  }

  stringstream prefix;
  prefix.str("");
  prefix << filename_prefix_ << "_" << id;
  chunk_ = new Chunk(prefix.str());
  chunk_id_ = id;
}

void Db::ResetChunk() {
  SetChunk(0);
}

bool Db::NextChunk() {
  if (chunk_id_ + 1< number_chunks_) {
    SetChunk(chunk_id_ + 1);
    return true;
  } else {
    chunk_id_ =  number_chunks_;
    return false;
  }
}
